@@ -15,13 +15,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with FIAT. If not, see <http://www.gnu.org/licenses/>.
 
-from . import finite_element, polynomial_set, dual_set, functional
+from __future__ import absolute_import, print_function, division
 
-class MorleyDualSet( dual_set.DualSet ):
+from FIAT import finite_element, polynomial_set, dual_set, functional
+
+
+class MorleyDualSet(dual_set.DualSet):
     """The dual basis for Lagrange elements.  This class works for
     simplices of any dimension.  Nodes are point evaluation at
     equispaced points."""
-    def __init__( self, ref_el ):
+
+    def __init__(self, ref_el):
         entity_ids = {}
         nodes = []
         cur = 0
@@ -34,42 +38,31 @@ class MorleyDualSet( dual_set.DualSet ):
         if sd != 2:
             raise Exception("Illegal spatial dimension")
 
-        pd = functional.PointDerivative
-
         # vertex point evaluations
 
         entity_ids[0] = {}
-        for v in sorted( top[0] ):
-            nodes.append( functional.PointEvaluation( ref_el, verts[v] ) )
+        for v in sorted(top[0]):
+            nodes.append(functional.PointEvaluation(ref_el, verts[v]))
 
             entity_ids[0][v] = [cur]
             cur += 1
-                          
+
         # edge dof -- normal at each edge midpoint
         entity_ids[1] = {}
-        for e in sorted( top[1] ):
-            pt = ref_el.make_points( 1, e, 2 )[0]
-            n = functional.PointNormalDerivative( ref_el, e, pt )
-            nodes.append( n )
+        for e in sorted(top[1]):
+            pt = ref_el.make_points(1, e, 2)[0]
+            n = functional.PointNormalDerivative(ref_el, e, pt)
+            nodes.append(n)
             entity_ids[1][e] = [cur]
             cur += 1
 
-        dual_set.DualSet.__init__( self, nodes, ref_el, entity_ids )
+        super(MorleyDualSet, self).__init__(nodes, ref_el, entity_ids)
 
-class Morley( finite_element.FiniteElement ):
+
+class Morley(finite_element.CiarletElement):
     """The Morley finite element."""
-    def __init__( self, ref_el ):
-        poly_set = polynomial_set.ONPolynomialSet( ref_el, 2 )
-        dual = MorleyDualSet( ref_el  )
-        finite_element.FiniteElement.__init__( self, poly_set, dual, 2 )
-
-if __name__=="__main__":
-    from . import reference_element
-    T = reference_element.DefaultTriangle()
-    U = Morley( T )
-
-    Ufs = U.get_nodal_basis()
-    pts = T.make_lattice( 1 )
-    print(pts)
-    print(list(Ufs.tabulate(pts).values())[0])
 
+    def __init__(self, ref_el):
+        poly_set = polynomial_set.ONPolynomialSet(ref_el, 2)
+        dual = MorleyDualSet(ref_el)
+        super(Morley, self).__init__(poly_set, dual, 2)
